/*
 * Copyright 2011 Raster Software Vigo
 *
 * This file is part of ttos
 *
 * ttos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ttos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * clock.c
 *
 *  Created on: 16/09/2010
 *      Author: raster
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <locale.h>
#include <libintl.h>

#define _(string) gettext (string)

#include "../ttos_lib/ttos.h"
#include "../minibus/minibus.h"

//#define BUS_TEST

struct ttos_object *master;
struct ttos_registry *registry;
uint32_t tab_id;
bool first_time;
int32_t added_days;
uint8_t date_format;

uint32_t port_sb;
uint32_t port_sb2;


void paint_date(uint32_t tab,int16_t x, int16_t y, time_t hour) {

	struct tm *local_time;
	uint8_t string[256];

	local_time=localtime(&hour);
	x*=master->width;
	y*=master->height;

	switch(date_format) {
	case 0: // month (letters), day, year
		strftime(string,100,"%A %B, %e %Y",local_time);
	break;
	case 1:// month (number), day, year
		strftime(string,100,"%A %m/%d/%Y",local_time);
	break;
	case 2: // day, month (letters), year
			strftime(string,100,"%A %e %B %Y",local_time);
	break;
	case 3:// day, month (number), year
		strftime(string,100,"%A %e/%m/%Y",local_time);
	break;
	default:
		strftime(string,100,"%A %B, %e %Y",local_time);
	break;
	}

	ttos_new_label(master,TTOS_SYNC_ANSWER,tab,x+0,y+(3*master->height/5),master->width,2*master->height/5,string,NULL,TTOS_WIDGET_FONT_HALIGN_CENTER,0);

}

uint32_t paint_clock() {

	uint32_t old_tab,newtab,x,count,widget,inc;
	struct tm *local_time;
	time_t hour;
	uint8_t ascii_time[6],path[100],*pos;
	struct ttos_color color;

	color.r=color.g=color.b=color.a=0xFF;

	old_tab=tab_id;
	newtab=ttos_new_tab(master,TTOS_SYNC_ANSWER);
	ttos_set_tab_bg(master,TTOS_SYNC_ANSWER,newtab,&color);
	hour=time(NULL);
	hour+=(added_days*86400);
	local_time=localtime(&hour);

	sprintf(ascii_time,"%02d:%02d",local_time->tm_hour,local_time->tm_min);

	x=(master->width-310)/2;
	for(count=0,pos=ascii_time;count<5;count++,pos++) {
		if (*pos==':') {
			sprintf(path,"sep.tga");
			inc=30;
		} else {
			sprintf(path,"%c.tga",*pos);
			inc=70;
		}
		widget=ttos_new_picture(master,TTOS_SYNC_ANSWER,newtab,x,(master->height/2)-100,path,TTOS_WIDGET_H_FIXED|TTOS_WIDGET_V_FIXED,0);
		x+=inc;
	}

	paint_date(newtab,0,-1,hour-604800);
	paint_date(newtab,-1,0,hour-86400);
	paint_date(newtab,0,0,hour);
	paint_date(newtab,1,0,hour+86400);
	paint_date(newtab,0,1,hour+604800);

	ttos_set_tab_size(master,TTOS_SYNC_ANSWER,newtab,-(master->width),-(master->height),3*master->width,3*master->height,true);

	tab_id=newtab;
	ttos_set_visible_tab(master,TTOS_SYNC_ANSWER,newtab,TTOS_TAB_SWAP_MODE_REPLACE, TTOS_NO_ANSWER);
	if (old_tab!=0) {
		ttos_destroy_tab(master,TTOS_SYNC_ANSWER,old_tab,0);
	}

	ttos_repaint_screen(master,TTOS_SYNC_ANSWER);
	return (60-local_time->tm_sec);
}

uint32_t timer_callback(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	paint_clock();
	if (first_time) {
		ttos_create_timer(master,TTOS_SYNC_ANSWER,60000,10,true);
		first_time=false;
	}

#ifdef BUS_TEST
	minibus_start_message(master->conn,port_sb2,master->ipc_uid);
	minibus_write_long(master->conn,TTOS_SR_BUS_MESSAGE);
	minibus_write_string(master->conn,"Test clock");
	minibus_write_long(master->conn,0x12345678);
	minibus_send(master->conn);
	ttos_wait_for_error(master,master->ipc_uid);
#endif
	return 0;
}

#ifdef BUS_TEST
uint32_t bus_callback(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	struct minibus_data_element data;
	uint32_t retval;

	printf("Clock: received bus1 callback\n");
	while(1) {
		retval=minibus_read_data(conn,&data);
		if (retval) {
			break;
		}
		ttos_print_type(&data);
	}
	printf("\n");

}
#endif

uint32_t scroll_callback(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	paint_clock();
	return 0;
}

void main_callback(struct ttos_object *object, uint32_t event, void *user_data) {

	struct minibus_data_element *data;
	int32_t error;
	int16_t xoffset,yoffset,ratio;

	if (!master->fg) {
		minibus_exit_loop(object->queue);
		return;
	}

	if (event==TTOS_SR_TAB_DRAG) {
		data=minibus_read_params(object->conn,"ssssss",&error);
		if (!data) {
			return;
		}
		xoffset=data[0].value16;
		yoffset=data[1].value16;
		free(data);
		if (yoffset) {
			ratio=xoffset/yoffset;
		} else {
			ratio=1;
		}
		if (ratio) {
			if (xoffset>0) {
				ttos_set_tab_offset(master,TTOS_SYNC_ANSWER,tab_id,1,0,11,TTOS_TAB_OFFSET_SCREEN_SIZE_COORDS|TTOS_TAB_OFFSET_DO_SCROLL);
				added_days-=1;
			} else {
				ttos_set_tab_offset(master,TTOS_SYNC_ANSWER,tab_id,-1,0,11,TTOS_TAB_OFFSET_SCREEN_SIZE_COORDS|TTOS_TAB_OFFSET_DO_SCROLL);
				added_days+=1;
			}
		} else {
			if (yoffset>0) {
				ttos_set_tab_offset(master,TTOS_SYNC_ANSWER,tab_id,0,1,11,TTOS_TAB_OFFSET_SCREEN_SIZE_COORDS|TTOS_TAB_OFFSET_DO_SCROLL);
				added_days-=7;
			} else {
				ttos_set_tab_offset(master,TTOS_SYNC_ANSWER,tab_id,0,-1,11,TTOS_TAB_OFFSET_SCREEN_SIZE_COORDS|TTOS_TAB_OFFSET_DO_SCROLL);
				added_days+=7;
			}
		}
	}
}


int main(int argc,char **argv) {

	struct minibus_connection *conn;
	struct ttos_color tcolor,bgcolor;
	struct ttos_registry_element *relement;
	uint32_t timeto;

	setlocale (LC_ALL, "");
	bindtextdomain ("ttos_clock", NULL);
	textdomain ("ttos_clock");

	added_days=0;

	registry=ttos_new_registry("clock");
	relement=ttos_registry_retrieve_key(registry,"date_format");
	if (!relement) {
		date_format=0;
	} else {
		date_format=relement->value.value8;
	}
	ttos_destroy_registry(registry);

	master=ttos_init_system(0,argc,argv);
	tab_id=0;

	if (master==NULL) {
		printf("Can't connect to the queue. Aborting\n");
		exit(1);
	}

	master->cb_port0=main_callback;
	minibus_add_callback(master->queue,10,timer_callback,NULL);
	minibus_add_callback(master->queue,11,scroll_callback,NULL);

#ifdef BUS_TEST
	port_sb=ttos_subscribe_to_bus(master,TTOS_SYNC_ANSWER,"test_bus1");
	port_sb2=ttos_subscribe_to_bus(master,TTOS_SYNC_ANSWER,"test_bus2");
	minibus_add_callback(master->queue,port_sb,bus_callback,NULL);
#endif

	timeto=paint_clock();
	first_time=true;
	ttos_create_timer(master,TTOS_SYNC_ANSWER,1000+timeto*1000,10,false);

	minibus_run_loop(master->queue);

	ttos_destroy_system(master);

	return 0;
}
