/*
 * Copyright 2010 (C) Raster Software Vigo (Sergio Costas)
 * This file is part of MINIBUS
 *
 * MINIBUS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * MINIBUS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*
 * test.c
 *
 *  Created on: 21/08/2010
 *      Author: raster
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <poll.h>

#include "minibus.h"

void print_data(struct minibus_data_element *data) {

	switch (data->type) {
	case MINIBUS_TYPE_INT8:
		printf("uint8_t %d ",data->value8);
	break;
	case MINIBUS_TYPE_INT16:
		printf("uint16_t %d ",data->value16);
	break;
	case MINIBUS_TYPE_INT32:
		printf("uint32_t %d ",data->value32);
	break;
	case MINIBUS_TYPE_STRING:
		printf("string (%d) '%s' ",data->stringsize,data->string);
	break;
	case MINIBUS_TYPE_BLOB:
		printf("BLOB (%d) '%s' ",data->blobsize,data->blob);
	break;
	default:
	break;
	}
}


int32_t cbtimeout(struct minibus_master *object,uint32_t timeout, void *data) {

	if (timeout==1) {
		minibus_exit_loop(object);
		printf("Exiting\n");
	}

	return(6000);
}

uint32_t cbtest(uint32_t port, uint32_t answer_port, struct minibus_connection *connection,void *data) {

	printf("Callback de prueba, en conexion %d, puerto %d, respuesta en %d, con datos %lld\n",connection->id_socket,port,answer_port,(long long int)data);
	return 0;
}

uint32_t cbtest_showall(uint32_t port, uint32_t answer_port, struct minibus_connection *connection,void *data) {

	struct minibus_data_element data2;

	printf("Recibido mensaje en puerto %d con datos:\n",port);

	while(-2!=minibus_read_data(connection,&data2)) {
		print_data(&data2);
	}
	printf("\n");
	return 0;

}


uint32_t cbtestdf(uint32_t port, uint32_t answer_port, struct minibus_connection *connection,void *data) {

	struct minibus_master *cola=data;

	printf("Callback por defecto, en conexion %d, puerto %d, respuesta en %d, con datos %lld\n",connection->id_socket,port,answer_port,(long long int)data);
	if (port==435) {
		printf("Envio en callback de prueba la cadena HOLA MUNDO y el long 77843, modo %d\n",connection->s_mode);
		minibus_write_string(cola->send_connection,"HOLA MUNDO");
		minibus_write_long(cola->send_connection,77843);
		minibus_server_send(cola->send_connection,connection,false);
	}
	return 0;
}


uint32_t nueva_conexion(struct minibus_connection *connection,void *data) {

	printf("Se ha hecho una nueva conexion %d con datos %lld\n",connection->id_socket,(long long int)data);
	return 0;

}

uint32_t cierra_conexion(struct minibus_connection *connection,void *data) {

	printf("Se ha cerrado una nueva conexion %d con datos %lld\n",connection->id_socket,(long long int)data);
	return 0;
}


void test_listener_callback (int fd, short revents,void *data) {

	char data2[16];
	int length;

	length=read(fd,data2,16);
	printf("Received from listener %d bytes (%s)\n",length,data2);

}

void print_list(struct minibus_list *lista_test) {

	struct minibus_list_element *loop;

	for(loop=lista_test->list;loop!=NULL;loop=loop->next) {
		printf("Element: %lld\n",(long long int)loop->data);
	}
	printf("\n");
	for(loop=lista_test->last;loop!=NULL;loop=loop->prev) {
		printf("Element: %lld\n",(long long int)loop->data);
	}
	printf("\n");
}

bool number_compare(void *v1, void *v2) {

	printf("%lld y %lld %d\n",(long long int)v1,(long long int)v2,v1<=v2);
	return (v1<=v2);

}

int main(int argc,char **argv) {

	struct minibus_master *cola;
	struct minibus_message_element *element;
	struct minibus_connection *conn;
	struct minibus_data_element data;
	int pipefd[2];

	struct minibus_list lista_test;


	/*uint8_t *cadena;
	uint8_t v1;
	uint16_t v2;
	uint32_t v3;*/
	int i;

	if (argc>3) {
		printf("No hay tantos tests\n");
		exit(1);
	}

	if (argc==3) {
		printf("Empiezo de nuevo\n");
		if (0==fork()) {
			//hijo
			sleep(1);
			cola=minibus_init("/tmp/prueba",0);
			if (!cola) {
				printf("Error 1\n");
				exit(1);
			}
			conn=minibus_get_first_connection(cola);
			minibus_start_message(conn,0,0);
			minibus_write_long(conn,0);
			minibus_send(conn);

			minibus_start_message(conn,0,0);
			minibus_write_long(conn,0);
			minibus_send(conn);

			minibus_start_message(conn,0,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_string(conn,"crawler.tga");
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_send(conn);

			minibus_start_message(conn,0,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_string(conn,"En un lugar de la mancha, de cuyo nombre no quiero acordarme,\nha tiempo que vivia un hidalgo de los de adarga antigua, lanza en astillero, rocin flaco, y\ngalgo corredor.");
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_send(conn);

			minibus_start_message(conn,0,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_string(conn,"");
			minibus_write_string(conn,"Un boton");
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_send(conn);

			minibus_start_message(conn,0,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_string(conn,"");
			minibus_write_string(conn,"Salir");
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_write_long(conn,0);
			minibus_send(conn);

			sleep(1);
		} else {
			cola=minibus_init("/tmp/prueba",1);
			if (!cola) {
				printf("Error 0\n");
				exit(1);
			}
			conn=minibus_get_first_connection(cola);
			minibus_add_callback(cola,0,cbtest_showall,NULL);
			minibus_set_timeout_callback(cola,cbtimeout,NULL);
			minibus_run_loop(cola);
		}
		minibus_release(cola);
		exit(0);
	}

	if (argc==2) {
		printf("Prueba de orden\n");
		minibus_list_init(&lista_test,NULL,number_compare);
		minibus_list_add_sorted(&lista_test,(void *)5);
		print_list(&lista_test);
		minibus_list_add_sorted(&lista_test,(void *)10);
		print_list(&lista_test);
		minibus_list_add_sorted(&lista_test,(void *)5);
		print_list(&lista_test);
		minibus_list_add_sorted(&lista_test,(void *)15);
		print_list(&lista_test);
		minibus_list_add_sorted(&lista_test,(void *)20);
		print_list(&lista_test);
		minibus_list_add_sorted(&lista_test,(void *)10);
		print_list(&lista_test);
		minibus_list_add_sorted(&lista_test,(void *)12);
		print_list(&lista_test);
		minibus_list_add_sorted(&lista_test,(void *)6);
		print_list(&lista_test);
		minibus_list_add_sorted(&lista_test,(void *)3);
		print_list(&lista_test);
		minibus_list_add_sorted(&lista_test,(void *)0);
		print_list(&lista_test);
		minibus_list_free(&lista_test);
		return 0;
	}

	pipe(pipefd);
	if (0==fork()) {
		// hijo

		i=fork();
		sleep(1);
		cola=minibus_init("/tmp/prueba",0);
		if (cola==NULL) {
			printf("El hijo no puede conectarse a la cola\n");
		}
		if (i) {
			sleep(3);
		}

		/*minibus_write_string(cola->conections.list->data,"Hola mundo");
		minibus_write_blob(cola->conections.list->data,"Hola a todos",13);
		minibus_write_byte(cola->conections.list->data,5);
		minibus_write_short(cola->conections.list->data,35627);
		minibus_write_long(cola->conections.list->data,8898383);
		minibus_send(cola->conections.list->data);*/

		conn=minibus_get_first_connection(cola);
		minibus_write_long(conn,4463+i); // message type
		minibus_write_long(conn,20); // answer port
		element=minibus_create_send_message_element(conn);
		minibus_list_add(&conn->send_buffer,element);

		minibus_write_long(conn,35627+i);
		minibus_write_long(conn,30); // answer port
		minibus_send(conn);
		minibus_write_long(conn,435+i);
		minibus_write_long(conn,40); // answer port
		minibus_send(conn);
		write(pipefd[1],"hola",5);
		if (i==0) {
			printf("Esperando datos del callback\n");
			minibus_receive(conn,0);
			printf("Recibidos datos del callback\n");
			while(-2!=minibus_read_data(conn,&data)) {
				print_data(&data);
			}
			printf("\n");
			//cadena=minibus_read_string(cola->conections.list->data);
			//valor=minibus_read_long(cola->conections.list->data);
			//printf("Recibido string %s y valor %d\n",cadena,valor);
		}
		minibus_write_long(conn,2+i);
		minibus_write_long(conn,50); // answer port
		minibus_send(conn);
		if (!i) {
			sleep(2);
		}
		minibus_release(cola);
		return 0;
	} else {
		// padre
		cola=minibus_init("/tmp/prueba",1);
		if (cola==NULL) {
			printf("El padre no puede crear la cola\n");
		}

		minibus_set_new_connection_callback(cola,nueva_conexion,(void *)1);
		minibus_set_closed_connection_callback(cola,cierra_conexion,(void *)2);
		minibus_set_default_callback(cola,cbtestdf,cola);
		minibus_set_timeout_callback(cola,cbtimeout,NULL);
		minibus_add_callback(cola,2,cbtest,(void *)4);
		minibus_add_callback(cola,435,cbtest,(void *)5);

		minibus_remove_callback(cola,435);

		minibus_add_listener(cola,pipefd[0],POLLIN,test_listener_callback,NULL);
		/*retval=minibus_accept(cola);
		printf("Accept: %d\n",retval);
		minibus_receive(cola->conections.list->data,0);
		cadena=minibus_read_string(cola->conections.list->data);
		printf("Cadena: %s\n",cadena);
		cadena=minibus_read_blob(cola->conections.list->data,13);
		printf("Blob: %s\n",cadena);
		v1=minibus_read_byte(cola->conections.list->data);
		printf("v1: %d\n",v1);
		v2=minibus_read_short(cola->conections.list->data);
		printf("v2: %d\n",v2);
		v3=minibus_read_long(cola->conections.list->data);
		printf("v3: %d\n",v3);*/
		minibus_run_loop(cola);


	}
	sleep(1);
	printf("Libero local\n");
	minibus_release(cola);

	return 0;
}

