/*
 * Copyright 2011 Raster Software Vigo
 *
 * This file is part of ttos
 *
 * ttos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ttos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * set config.c
 *
 *  Created on: 16/09/2010
 *      Author: raster
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <locale.h>
#include <libintl.h>
#include <string.h>

#include <linux/rtc.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define _(string) gettext (string)

#include "../ttos_lib/ttos.h"
#include "../minibus/minibus.h"

struct ttos_object *master;
uint32_t tab_id;
uint32_t *uids;

void main_callback(struct ttos_object *object, uint32_t event, void *user_data) {

	if (!master->fg) {
		minibus_exit_loop(object->queue);
		return;
	}
}

uint32_t set_date_format(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	uint32_t tab;
	uint32_t uid;
	uint32_t rv;
	struct ttos_color color;
	struct _ttos_key_buttons buttons[]={
		{TTOS_NO_STOCK_BUTTON,"month (letters), day, year",NULL,true,13,0,0},
		{TTOS_NO_STOCK_BUTTON,"month (numbers), day, year",NULL,true,13,0,0},
		{TTOS_NO_STOCK_BUTTON,"day, month (letters), year",NULL,true,13,0,0},
		{TTOS_NO_STOCK_BUTTON,"day, month (numbers), year",NULL,true,13,0,0},
	};

	struct ttos_color color2[]={
				{0xFF,0xFF,0xFF,0xFF}};

	struct ttos_keyboard s_keyb;

	tab=ttos_new_tab(master,TTOS_SYNC_ANSWER);
	color.r=color.g=color.b=color.a=0xFF;
	ttos_set_tab_bg(master,TTOS_SYNC_ANSWER,tab,&color);

	s_keyb.buttons=buttons;
	s_keyb.colors=color2;
	s_keyb.width=master->width-8;
	s_keyb.height=36;

	uids=ttos_paint_grid_buttons(master,tab,0,true,4,&s_keyb,true);

	rv=ttos_set_visible_tab(master,TTOS_SYNC_ANSWER,tab,TTOS_TAB_SWAP_MODE_SCROLL_RIGHT,TTOS_RESERVED_PORTS+2);

	if ((TTOS_ERROR_NO_ERROR==rv)||(TTOS_ERROR_DESTROY_DELAYED==rv)) {
		minibus_wait_for_port(master->conn,TTOS_RESERVED_PORTS+2,false);
	} else {
		ttos_print_error(rv,true);
	}
	ttos_destroy_tab(master,TTOS_SYNC_ANSWER,tab,TTOS_TAB_DELAY_DESTROY);
	ttos_repaint_screen(master,TTOS_SYNC_ANSWER);
}


int32_t show_main_menu() {

	uint32_t tab;
	uint32_t uid;
	uint32_t rv;
	struct ttos_color color;
	struct _ttos_key_buttons buttons[]={
		{TTOS_NO_STOCK_BUTTON,"Set date/time",NULL,true,10,0,0},
		{TTOS_NO_STOCK_BUTTON,"Set date format",NULL,true,11,0,0},
		{TTOS_NO_STOCK_BUTTON,"About TTOS",NULL,true,14,0,0},
		{TTOS_EXIT_BUTTON,NULL,NULL,true,12,0,0} };

	struct ttos_color color2[]={
				{0xFF,0xFF,0xFF,0xFF}};

	struct ttos_keyboard s_keyb;

	tab=ttos_new_tab(master,TTOS_SYNC_ANSWER);
	color.r=color.g=color.b=color.a=0xFF;
	ttos_set_tab_bg(master,TTOS_SYNC_ANSWER,tab,&color);

	s_keyb.buttons=buttons;
	s_keyb.colors=color2;
	s_keyb.width=master->width-8;
	s_keyb.height=36;

	ttos_paint_grid_buttons(master,tab,0,true,4,&s_keyb,false);

	rv=ttos_set_visible_tab(master,TTOS_SYNC_ANSWER,tab,TTOS_TAB_SWAP_MODE_SCROLL_RIGHT,TTOS_RESERVED_PORTS+2);

	if ((TTOS_ERROR_NO_ERROR==rv)||(TTOS_ERROR_DESTROY_DELAYED==rv)) {
		minibus_wait_for_port(master->conn,TTOS_RESERVED_PORTS+2,false);
	} else {
		ttos_print_error(rv,true);
	}
	ttos_destroy_tab(master,TTOS_SYNC_ANSWER,tab,TTOS_TAB_DELAY_DESTROY);
	ttos_repaint_screen(master,TTOS_SYNC_ANSWER);
}


uint32_t about_ttos(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	char text[512];
	struct ttos_dialog_button buttons = {TTOS_OK_BUTTON , NULL, NULL, 0};

	sprintf(text,"TTOS\n\nLightweight Compositing Window Manager\n\nVersion %s\n\n(C) 2011-2012 Raster Software Vigo\nhttp://www.rastersoft.com",TTOS_VERSION_NUMBER);

	ttos_show_dialog(master,text,TTOS_WIDGET_FONT_ALIGN_CENTER|TTOS_WIDGET_FONT_MULTILINE|TTOS_WIDGET_FONT_WORD_CUT,&buttons,1);
	show_main_menu();

	return 0;
}


uint32_t set_time(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	struct rtc_time rtctime;
	struct tm swtime;
	struct timeval final_time;
	int rtc_dev;
	int32_t retval;
	bool leap_year;
	int32_t max_day;

	rtc_dev=open("/dev/rtc",O_RDONLY);
	if (rtc_dev<0) {
		struct ttos_dialog_button buttons = {TTOS_OK_BUTTON , NULL, NULL, 0};
		ttos_show_dialog(master,_("Can't get access to the clock"),TTOS_WIDGET_FONT_ALIGN_CENTER,&buttons,1);
		show_main_menu();
		return 0;
	}
	memset(&rtctime,0,sizeof(struct rtc_time));
	retval=ioctl(rtc_dev,RTC_RD_TIME,&rtctime);
	if (retval==-1) {
		struct ttos_dialog_button buttons = {TTOS_OK_BUTTON , NULL, NULL, 0};
		ttos_show_dialog(master,_("Can't get access to the clock"),TTOS_WIDGET_FONT_ALIGN_CENTER,&buttons,1);
		close(rtc_dev);
		show_main_menu();
		return 0;
	}

	retval=ttos_ask_for_digit(master,"Year",4,rtctime.tm_year+1900,1900,2100,TTOS_RIGHT_BUTTON);
	if (retval<0) {
		close(rtc_dev);
		show_main_menu();
		return 0;
	}

	if (retval%4) {
		leap_year=false;
	} else {
		if (retval%100) {
			leap_year=true;
		} else {
			if (retval%400) {
				leap_year=false;
			} else {
				leap_year=true;
			}
		}
	}

	rtctime.tm_year=retval-1900;
	retval=ttos_ask_for_digit(master,"Month",2,rtctime.tm_mon+1,1,12,TTOS_RIGHT_BUTTON);
	if (retval<0) {
		close(rtc_dev);
		show_main_menu();
		return 0;
	}
	rtctime.tm_mon=retval-1;
	switch (retval) {
	case 1:
	case 3:
	case 5:
	case 7:
	case 8:
	case 10:
	case 12:
		max_day=31;
	break;
	case 2: // february
		if (leap_year) {
			max_day=29;
		} else {
			max_day=28;
		}
	break;
	default:
		max_day=30;
	break;
	}
	retval=ttos_ask_for_digit(master,"Day",2,rtctime.tm_mday,1,max_day,TTOS_RIGHT_BUTTON);
	if (retval<0) {
		close(rtc_dev);
		show_main_menu();
		return 0;
	}
	rtctime.tm_mday=retval;
	retval=ttos_ask_for_digit(master,"Hour",2,rtctime.tm_hour,0,23,TTOS_RIGHT_BUTTON);
	if (retval<0) {
		close(rtc_dev);
		show_main_menu();
		return 0;
	}
	rtctime.tm_hour=retval;
	retval=ttos_ask_for_digit(master,"Minutes",2,rtctime.tm_min,0,59,TTOS_OK_BUTTON);
	if (retval<0) {
		close(rtc_dev);
		show_main_menu();
		return 0;
	}
	rtctime.tm_min=retval;

	retval=ioctl(rtc_dev,RTC_SET_TIME,&rtctime);
	close(rtc_dev);
	swtime.tm_sec=rtctime.tm_sec;
	swtime.tm_min=rtctime.tm_min;
	swtime.tm_hour=rtctime.tm_hour;
	swtime.tm_mday=rtctime.tm_mday;
	swtime.tm_mon=rtctime.tm_mon;
	swtime.tm_year=rtctime.tm_year;
	final_time.tv_sec=mktime(&swtime);
	final_time.tv_usec=0;
	settimeofday(&final_time,NULL);

	show_main_menu();
	return 0;
}

uint32_t quit(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	minibus_exit_loop(master->queue);

	return 0;
}

uint32_t select_date_format(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	uint32_t loop;
	int32_t error;
	struct minibus_data_element *data;
	int8_t value;
	struct ttos_registry *registry;
	struct minibus_data_element lvalue;

	if (uids) {
		value=-1;
		data=minibus_read_params(master->conn,"lblllll",&error);
		if (data) {
			for(loop=0;loop<4;loop++) {
				if (uids[loop]==data[2].value32) {
					value=loop;
					break;
				}
			}
			if (value>=0) {
				lvalue.type=MINIBUS_TYPE_INT8;
				lvalue.value8=value;
				registry=ttos_new_registry("clock");
				if (0==ttos_registry_insert_key(registry,"date_format",&lvalue)) {
					ttos_store_registry(registry);
				}
				ttos_destroy_registry(registry);
			}
			free(data);
		}
		free(uids);
	}
	uids=NULL;
	show_main_menu();
}

int main(int argc,char **argv) {


	setlocale (LC_ALL, "");
	bindtextdomain ("ttos_setclock", NULL);
	textdomain ("ttos_setclock");

	master=ttos_init_system(0,argc,argv);

	if (master==NULL) {
		printf("Can't connect to the queue. Aborting\n");
		exit(1);
	}

	master->cb_port0=main_callback;

	minibus_add_callback(master->queue,10,set_time,NULL);
	minibus_add_callback(master->queue,11,set_date_format,NULL);
	minibus_add_callback(master->queue,12,quit,NULL);
	minibus_add_callback(master->queue,13,select_date_format,NULL);
	minibus_add_callback(master->queue,14,about_ttos,NULL);

	show_main_menu();

	minibus_run_loop(master->conn->master_queue);

	ttos_destroy_system(master);

	return 0;
}
