/*
 * Copyright 2011 Raster Software Vigo
 *
 * This file is part of ttos
 *
 * ttos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ttos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * terminal.c
 *
 *  Created on: 15/03/2011
 *      Author: raster
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libintl.h>
#include <signal.h>
#include <unistd.h>

#include "../ttos_lib/ttos.h"
#include "../minibus/minibus.h"
#include "terminal.h"

#define _(string) gettext (string)

extern char **environ;

struct ttos_object *master;
int pipe_stdin[2];
int pipe_stdout[2];
Dstring to_print;
uint32_t screen;
uint32_t base_tab;
int32_t child_pid;

void kill_handler(int v) {

	minibus_exit_loop(master->conn->master_queue);

}

void exit_cb(struct ttos_object *master,void *data){

	// kill the shell and all its children process, thanks to the setsid in ttos_launch_child that groups them in a single group
	kill(-child_pid,SIGKILL);
}

void refresh_text() {

	uint32_t counter,counter2;
	Cstring loop,loop2;

	for(loop=to_print->text+to_print->size,counter=0;(loop!=to_print->text)&&(counter<TERMINAL_MAX_LINES);loop--) {
		if ((*loop)=='\n') {
			counter++;
		}
	}
	if ((counter>=TERMINAL_MAX_LINES)&&(loop!=to_print->text)) {
		loop++;
		for(counter2=0,loop2=to_print->text;(*loop);counter2++) {
			*(loop2++)=*(loop++);
			*(loop2)=0;
		}
		to_print->size=counter2;
	}

	ttos_set_label_text(master,TTOS_SYNC_ANSWER,screen,to_print->text,TTOS_WIDGET_CLICK_EVENT|TTOS_WIDGET_FONT_HALIGN_LEFT|TTOS_WIDGET_FONT_VALIGN_TOP|TTOS_WIDGET_FONT_MULTILINE|TTOS_WIDGET_FONT_UPDATE_SIZE);
	ttos_set_tab_size(master,TTOS_SYNC_ANSWER,base_tab,0,0,master->width,master->height,false);
	ttos_repaint_screen(master,TTOS_SYNC_ANSWER);

}

void available_data_cb (int fd, short revents,void *data) {

	char tmp[1025];
	ssize_t length;

	if (!(revents|POLLIN)) {
		return;
	}
	length=read(fd,tmp,1024);
	if (length<0) {
		return;
	}

	tmp[length]=0;
	minibus_append_dstring(to_print,tmp);
	refresh_text();
}

uint32_t click_cb(uint32_t port, uint32_t answer_port, struct minibus_connection *connection,void *user_data) {

	Cstring command;
	uint32_t rv;

	command=ttos_keyboard_intro(master,"");
	rv=ttos_set_visible_tab(master,TTOS_SYNC_ANSWER,base_tab,TTOS_TAB_SWAP_MODE_SCROLL_DOWN,TTOS_RESERVED_PORTS+2);

	if ((TTOS_ERROR_NO_ERROR==rv)||(TTOS_ERROR_DESTROY_DELAYED==rv)) {
		minibus_wait_for_port(master->conn,TTOS_RESERVED_PORTS+2,false);
	} else {
		ttos_print_error(rv,true);
	}
	ttos_repaint_screen(master,TTOS_SYNC_ANSWER);

	if (!command) {
		return 0;
	}

	write(pipe_stdin[1],command,strlen(command));
	write(pipe_stdin[1],"\n",1);
	refresh_text();
	free(command);
	return 0;
}

int main(int argc,char **argv) {

	Cstring text,command[3];
	int pipe_child[2];
	struct sigaction sg;
	struct ttos_color color;

	memset(&sg,0,sizeof(struct sigaction));
	sg.sa_handler=kill_handler;
	sigaction(SIGCHLD,&sg,NULL);

	to_print=minibus_new_dstring(_("User shell\nTouch the screen to show the keyboard.\n\n"));

	master=ttos_init_system(0,argc,argv);
	text=NULL;

	if (master==NULL) {
		printf("Can't connect to the queue. Aborting\n");
		exit(1);
	}

	master->exit_cb=exit_cb;
	pipe(pipe_stdin);
	pipe(pipe_stdout);

	color.r=0x00;
	color.g=0x00;
	color.b=0x00;
	color.a=0xFF;

	base_tab=ttos_new_tab(master,TTOS_SYNC_ANSWER);
	ttos_set_tab_bg(master,TTOS_SYNC_ANSWER,base_tab,&color);
	ttos_new_color_area(master,TTOS_SYNC_ANSWER,base_tab,0,0,master->width,master->height,&color,TTOS_WIDGET_CLICK_EVENT|TTOS_WIDGET_FIXED,10);

	color.r=0xFF;
	color.g=0xFF;
	color.b=0xFF;
	color.a=0xFF;

	screen=ttos_new_label(master,TTOS_SYNC_ANSWER,base_tab,2,2,master->width-4,master->height-4,to_print->text,&color,TTOS_WIDGET_CLICK_EVENT|TTOS_WIDGET_FONT_HALIGN_LEFT|TTOS_WIDGET_FONT_VALIGN_TOP|TTOS_WIDGET_FONT_MULTILINE|TTOS_WIDGET_FONT_UPDATE_SIZE,10);
	ttos_set_visible_tab(master,TTOS_SYNC_ANSWER,base_tab,TTOS_TAB_SWAP_MODE_REPLACE, TTOS_NO_ANSWER);
	ttos_repaint_screen(master,TTOS_SYNC_ANSWER);

	pipe_child[0]=pipe_stdin[0];
	pipe_child[1]=pipe_stdout[1];
	command[0]="sh";
	command[1]="-i";
	command[2]=NULL;

	child_pid=ttos_launch_child(command,pipe_child);
	printf("PID: %d\n",child_pid);
	minibus_add_callback(master->queue,10,click_cb,NULL);

	minibus_add_listener(master->queue,pipe_stdout[0],POLLIN,available_data_cb,NULL);

	minibus_run_loop(master->queue);

	return 0;
}
