/*
 * Copyright 2011 Raster Software Vigo
 *
 * This file is part of ttos
 *
 * ttos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ttos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * test.c
 *
 *  Created on: 24/10/2010
 *      Author: raster
 */

#include <sys/time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include "blit.h"
#include "bitmaps.h"
#include "minibus/minibus.h"
#include "portable.h"
#include "fonts.h"
#include "global_vars.h"

#include "test.h"

static volatile bool run_test;
static uint16_t y_coord;

void test_alarm_handler(int s) {
	run_test=false;
}

void test_meter(TTOS_Surface *base, Cstring text, uint32_t (*test)()) {

	uint32_t counter,fps;
	struct timeval tv,tv2;
	time_t v1,v2;
	TTOS_Surface *label=NULL;
	Cstring text2;
	struct ttos_color color;

	color.r=0;
	color.g=0;
	color.b=0;
	color.a=255;

	run_test=true;
	text2=malloc(200);
	alarm(5);
	gettimeofday(&tv,NULL);
	counter=test();
	gettimeofday(&tv2,NULL);
	v1=tv2.tv_sec-tv.tv_sec;
	if (tv2.tv_usec<tv.tv_usec) {
		v1--;
		v2=1000000+tv2.tv_usec-tv.tv_usec;
	} else {
		v2=tv2.tv_usec-tv.tv_usec;
	}

	v1*=1000;
	v1+=v2/1000;

	if (v1) {
		fps=((counter*1000)/v1);
	} else {
		fps=0;
	}
	sprintf(text2,"%s: %d r, %d ms, %d FPS",text,counter,(uint32_t)v1,fps);
	label=fonts_render_label(base_font,text2,0,0,&color);
	blit_blend(label,0,0,label->w,label->h,base,2,y_coord,0,0,base->w,base->h,false);
	y_coord+=label->h+2;
	blit_destroy_surface(label);
	ttos_dump_screen(base);
	free(text2);

}

uint32_t test_fonts() {

	struct ttos_color color;
	uint32_t counter;
	Cstring line;

	color.r=255;
	color.g=255;
	color.b=255;
	color.a=255;

	line="En un lugar de la mancha, de cuyo nombre no quiero acordarme";

	counter=0;
	while(run_test) {
		blit_destroy_surface(fonts_render_label(base_font,line,0,0,&color));
		counter++;
	}
	return(counter);
}

uint32_t test_blit() {

	uint32_t counter;
	TTOS_Surface *pantalla,*pantalla2;

	pantalla=bitmaps_load_TGA("pics/crawler2.tga");
	if(!pantalla) {
		return 0;
	}
	counter=0;
	pantalla2=blit_create_surface(320,240,true);
	while(run_test) {
		blit_blend(pantalla,0,0,160,120,pantalla2,0,0,0,0,320,240,false);
		counter++;
	}
	blit_destroy_surface(pantalla);
	blit_destroy_surface(pantalla2);
	return (counter);
}

uint32_t test_fill() {

	uint32_t counter;
	TTOS_Surface *pantalla;
	struct ttos_color color;

	color.r=253;
	color.g=253;
	color.b=253;
	color.a=129;

	counter=0;
	pantalla=blit_create_surface(320,240,true);
	while(run_test) {
		blit_fill(pantalla,0,0,320,240,&color,true);
		counter++;
	}
	blit_destroy_surface(pantalla);
	return (counter);
}

uint32_t test_dump_screen() {

	uint32_t counter;

	counter=0;
	while(run_test) {
		ttos_dump_screen(base);
		counter++;
	}
	return (counter);
}

void test_run_tests(TTOS_Surface *base,struct tsdev *ts) {

	struct sigaction sg;
	struct ttos_tssample sample;
	struct ttos_color color;

	memset(&sg,0,sizeof(struct sigaction));
	sg.sa_handler=test_alarm_handler;
	sigaction(SIGALRM,&sg,NULL);
	while(1) {
		y_coord=2;

		color.r=255;
		color.g=255;
		color.b=255;
		color.a=255;
		blit_fill(base,0,0,base->w,base->h,&color,true);
		ttos_dump_screen(base);

		//test_meter(base,"Fonts",test_fonts);
		//test_meter(base,"Blit",test_blit);
		test_meter(base,"Dump",test_dump_screen);
		//test_meter(base,"Fill",test_fill);

		do {
			ttos_read_ts(ts,&sample);
		} while(sample.type!=TTOS_TOUCH_CLICK);
	}
	exit(0);

}
