/*
 * Copyright 2010 Raster Software Vigo
 *
 * This file is part of ttos
 *
 * ttos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ttos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * ttos.h
 *
 *  Created on: 16/09/2010
 *      Author: raster
 */

#ifndef ATTOS_H_
#define ATTOS_H_

#include "../minibus/minibus.h"
#include "ttos_structs.h"
#include "helper_functions.h"
#include "ttos_keyboard.h"

#define TTOS_VERSION_NUMBER "1.4.0"

struct ttos_object * ttos_init_system (uint32_t flags,int argc,char **argv);
void ttos_destroy_system(struct ttos_object *object);

int32_t ttos_close_client(struct ttos_object *master, uint32_t answer_port, uint32_t app_uid);

int32_t ttos_define_windows(struct ttos_object *master,uint32_t answer_port,uint8_t nwindows, struct ttos_window_geometry windows[]);
int32_t ttos_set_window_visibility(struct ttos_object *master,uint32_t answer_port,uint8_t window, bool visible);

int32_t ttos_new_tab(struct ttos_object *master,uint32_t answer_port);
int32_t ttos_destroy_tab(struct ttos_object *master,uint32_t answer_port,uint32_t tab_uid, uint32_t flags);
int32_t ttos_set_visible_tab(struct ttos_object *master,uint32_t answer_port,uint32_t tab_uid,uint8_t swap_mode, uint32_t event_port);
int32_t ttos_get_visible_tab(struct ttos_object *master, uint32_t answer_port);
int32_t ttos_set_tab_bg(struct ttos_object *master,uint32_t answer_port,uint32_t tab_uid,struct ttos_color *color);
int32_t ttos_set_tab_offset (struct ttos_object *master,uint32_t answer_port, uint32_t tab_uid, int16_t new_x, int16_t new_y, uint32_t event_port, uint32_t Flags);
int32_t ttos_set_tab_size(struct ttos_object *master,uint32_t answer_port,uint32_t tab_uid,int16_t x, int16_t y, uint16_t w, uint16_t h, bool force);
int32_t ttos_set_tab_margins(struct ttos_object *master, uint32_t answer_port, uint32_t tab_uid, uint16_t margin_up, uint16_t margin_down, uint16_t margin_left, uint16_t margin_right);

int32_t ttos_repaint_screen(struct ttos_object *master,uint32_t answer_port);

int32_t ttos_create_timer(struct ttos_object *master,uint32_t answer_port, uint32_t timer_period,uint32_t cb_port,bool periodic);
int32_t ttos_delete_timer(struct ttos_object *master, uint32_t answer_port, uint32_t timer_uid);

int32_t ttos_set_fg_app (struct ttos_object *master, uint32_t answer_port, uint32_t app_uid);

int32_t ttos_set_client_window (struct ttos_object *master, uint32_t answer_port, uint32_t app_uid, uint8_t window_number);

uint32_t ttos_set_visibility (struct ttos_object *master, uint32_t answer_port, uint32_t widget_uid, bool visible);
uint32_t ttos_set_widget_position (struct ttos_object *master, uint32_t answer_port, uint32_t widget_uid, int16_t x, int16_t y);

uint32_t ttos_new_picture (struct ttos_object *master, uint32_t answer_port, uint32_t tab_uid, int16_t x, int16_t y,
						 Cstring filename, uint32_t flags, uint32_t event_port);
uint32_t ttos_set_picture (struct ttos_object *master, uint32_t answer_port, uint32_t picture_uid, Cstring filename);

uint32_t ttos_new_label (struct ttos_object *master, uint32_t answer_port, uint32_t tab_uid, int16_t x, int16_t y,
						 int16_t w, int16_t h, Cstring text, struct ttos_color *color, uint32_t flags, uint32_t event_port);
uint32_t ttos_set_label_text(struct ttos_object *master, uint32_t answer_port, uint32_t uid, Cstring text, uint32_t mask);

uint32_t ttos_new_button (struct ttos_object *master, uint32_t answer_port, uint32_t tab_uid, int16_t x, int16_t y,
						 int16_t w, int16_t h, enum ttos_stock_button stock_button, Cstring picture, Cstring text,
						 struct ttos_color *color, struct ttos_color *bgcolor, uint32_t flags, uint32_t event_port);

uint32_t ttos_new_color_area (struct ttos_object *master, uint32_t answer_port, uint32_t tab_uid, int16_t x, int16_t y,
						 int16_t w, int16_t h, struct ttos_color *color, uint32_t flags, uint32_t event_port);

uint32_t ttos_new_sharemem(struct ttos_object *master, uint32_t answer_port, uint16_t width, uint16_t height, bool has_alpha,
						   struct ttos_sharemem_object ** sharemem);
uint32_t ttos_destroy_sharemem (struct ttos_object *master, uint32_t answer_port, struct ttos_sharemem_object * sharemem);

uint32_t ttos_new_drawable (struct ttos_object *master, uint32_t answer_port, uint32_t tab_uid, int16_t x, int16_t y,
						 int16_t w, int16_t h, uint32_t flags, uint32_t event_port);

uint32_t ttos_new_drawable_from_sharemem (struct ttos_object *master, uint32_t answer_port, uint32_t tab_uid, int16_t x,
						int16_t y, struct ttos_sharemem_object * sharemem, uint32_t flags, uint32_t event_port);

uint32_t ttos_update_drawable (struct ttos_object *master, uint32_t answer_port, uint32_t widget_uid, uint32_t offset,
						 uint32_t length, uint8_t *data, uint32_t flags);

uint32_t ttos_update_sharemem (struct ttos_object *master, struct ttos_sharemem_object * sharemem, uint32_t offset,
						 uint32_t length, uint8_t *data, uint32_t flags);

enum ttos_file_type ttos_check_extension(Cstring fname);
Cstring ttos_get_upper_path(Cstring path, Cstring *tail);

uint32_t ttos_register_bus(struct ttos_object *master, uint32_t answer_port, Cstring bus_name, uint32_t flags);
uint32_t ttos_unregister_bus(struct ttos_object *master, uint32_t answer_port, Cstring bus_name);
uint32_t ttos_subscribe_to_bus(struct ttos_object *master, uint32_t answer_port, Cstring bus_name);
uint32_t ttos_unsubscribe_to_bus(struct ttos_object *master, uint32_t answer_port, Cstring bus_name);

#endif /* ATTOS_H_ */
