/*
 * Copyright 2011 Raster Software Vigo
 *
 * This file is part of ttos
 *
 * ttos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ttos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * ttos_structs.h
 *
 *  Created on: 26/01/2011
 *      Author: raster
 */

#ifndef TTOS_STRUCTS_H_
#define TTOS_STRUCTS_H_

#include <sys/types.h>
#include <stdint.h>
#include <stdbool.h>
#include "../minibus/minibus.h"

enum TTOS_PIXEL_FORMAT { TTOS_PIXEL_FORMAT_TOMTOM_RGB565, TTOS_PIXEL_FORMAT_OTHER };

enum TTOS_TOUCH_SCREEN_EVENTS  {TTOS_TS_NONE, TTOS_TS_CLICK, TTOS_TS_DOUBLE_CLICK, TTOS_TS_DRAG, TTOS_TS_END_DRAG, TTOS_TS_MASTER };

// TTOS commands. All starts at 0x80000000
enum TTOS_SYSTEM_COMMANDS  {TTOS_SM_INIT=0x80000000, TTOS_SM_DEFINE_WINDOWS, TTOS_SM_SET_CLIENT_WINDOW, TTOS_SM_WINDOW_SET_VISIBLE,
							TTOS_SM_MOVE_CLIENT_TO_FG,
							TTOS_SM_NEW_TAB, TTOS_SM_DESTROY_TAB, TTOS_SM_SELECT_TAB, TTOS_SM_SET_TAB_BG_COLOR, TTOS_SM_SET_TAB_OFFSET,
							TTOS_SM_SET_TAB_SIZE, TTOS_SM_SET_TAB_MARGINS, TTOS_SM_GET_CURRENT_TAB,
							TTOS_SM_NEW_SHAREMEM, TTOS_SM_DESTROY_SHAREMEM,
							TTOS_SM_NEW_WIDGET_PICTURE,TTOS_SM_SET_PICTURE,
							TTOS_SM_NEW_WIDGET_LABEL, TTOS_SM_SET_LABEL_TEXT,
							TTOS_SM_NEW_WIDGET_BUTTON, TTOS_SM_NEW_WIDGET_COLOR_AREA,
							TTOS_SM_NEW_WIDGET_DRAWABLE, TTOS_SM_UPDATE_DRAWABLE,
							TTOS_SM_SET_WIDGET_VISIBILITY, TTOS_SM_SET_WIDGET_POSITION,
							TTOS_SM_REPAINT_SCREEN, TTOS_SM_NEW_TIMER, TTOS_SM_DELETE_TIMER,
							TTOS_SM_CLOSE_CLIENT, TTOS_SM_REGISTER_BUS, TTOS_SM_UNREGISTER_BUS,
							TTOS_SM_SUBSCRIBE_BUS, TTOS_SM_UNSUBSCRIBE_BUS };

// TTOS answers. All starts at 0x80000000
enum TTOS_SYSTEM_RESPONSES {TTOS_SR_INIT=0x80000000, TTOS_SR_ERROR, TTOS_SR_NEW_RESOLUTION, TTOS_SR_NEW_CLIENT,
							TTOS_SR_DESTROYED_CLIENT, TTOS_SR_MOVED_TO_BG, TTOS_SR_MOVED_TO_FG, TTOS_SR_TOUCH_EVENT,
							TTOS_SR_EXIT, TTOS_SR_TIMER, TTOS_SR_SCROLL, TTOS_SR_TAB_DRAG, TTOS_SR_TAB_SWAP_END,
							TTOS_SR_NEW_WIDGET, TTOS_SR_UPDATED_BUS_USERS, TTOS_SR_BUS_MESSAGE, TTOS_SR_BUS_DESTROYED };

// Errors defined for TTOS_SR_ERROR. All starts at 0xC0000000
enum TTOS_SYSTEM_ERRORS {TTOS_ERROR_NO_ERROR=0xC0000000, TTOS_ERROR_ASYNC_RESPONSE, TTOS_ERROR_CANT_SEND_DATA,
						 TTOS_ERROR_CANT_READ_DATA, TTOS_ERROR_INCORRECT_FORMAT, TTOS_ERROR_INCORRECT_ANSWER_FORMAT,
						 TTOS_ERROR_CANT_LOAD_PICTURE, TTOS_ERROR_TOO_MANY_WIDGETS,
						 TTOS_ERROR_NO_MEMORY, TTOS_ERROR_INVALID_WINDOW_SIZE, TTOS_ERROR_RESTRICTED_COMMAND,
						 TTOS_ERROR_WINDOW_OUT_OF_RANGE, TTOS_ERROR_INVALID_UID,
						 TTOS_ERROR_INVALID_TAB_UID, TTOS_ERROR_NO_VISIBLE_TAB, TTOS_ERROR_CANT_CREATE_TAB,
						 TTOS_ERROR_DESTROY_DELAYED, TTOS_ERROR_INVALID_FLAGS,
						 TTOS_ERROR_FILE_NOT_FOUND, TTOS_ERROR_OLD_WINDOW_HAS_NO_FG,
						 TTOS_ERROR_INVALID_SWAP_MODE, TTOS_ERROR_INCORRECT_BLOB_SIZE, TTOS_ERROR_SHMEM_IN_USE,
						 TTOS_ERROR_INVALID_SHAREMEM_UID, TTOS_ERROR_SHAREMEM_HAS_NO_ALPHA, TTOS_ERROR_CANT_SHARE_MEM,
						 TTOS_ERROR_OFFSET_OUT_OF_RANGE, TTOS_ERROR_INVALID_OBJECT, TTOS_ERROR_BUS_ALREADY_REGISTERED,
						 TTOS_ERROR_CANT_RESERVE_UID, TTOS_ERROR_INVALID_BUS, TTOS_ERROR_NO_OWNER,
						 TTOS_ERROR_ALREADY_SUBSCRIBED, TTOS_ERROR_NOT_SUBSCRIBED, TTOS_ERROR_NOT_ALLOWED_TO_SEND,
						 TTOS_ERROR_INVALID_IPC_PORT, TTOS_ERROR_INVALID_IPC_ANSWER_PORT, TTOS_ERROR_INVALID_IPC_MESSAGE_FORMAT };

// Standard buttons available for dialogs
enum ttos_stock_button { TTOS_NO_STOCK_BUTTON=0, TTOS_OK_BUTTON, TTOS_CANCEL_BUTTON, TTOS_EXIT_BUTTON,
							 TTOS_LEFT_BUTTON, TTOS_RIGHT_BUTTON, TTOS_UP_BUTTON, TTOS_DOWN_BUTTON, TTOS_INTRO_BUTTON };

/*
 * Ports:
 *
 * 0x00000000: Reserved for system messages (TTOS_SR_NEW_RESOLUTION, TTOS_SR_MOVED_TO_BG, TTOS_SR_MOVED_TO_FG, TTOS_SR_NEW_CLIENT
											 TTOS_SR_DESTROYED_CLIENT, and TTOS_SR_TOUCH_EVENT for manager events)
 * 0x00000001 - 0x7FFFFFFF Available to user
 * 0x80000000 - 0xFFFFFFFF Reserved for TTOS_LIB when using synchronous calls
 */

#define TTOS_SYNC_ANSWER 0x00
#define TTOS_NO_ANSWER 0x00
#define TTOS_SYSTEM_PORT_1 0x00
#define TTOS_SYSTEM_PORT_2 0x80000000
#define TTOS_RESERVED_PORTS 0x80000001

// Masks for each possible UID
/* An UID type is defined by its four upper bits */
#define TTOS_WIDGET_UID_MASK 0x10000000
#define TTOS_TAB_UID_MASK 0x20000000
#define TTOS_TIMER_UID_MASK 0x30000000
#define TTOS_SHMEM_UID_MASK 0x40000000
#define TTOS_IPC_UID_MASK 0x80001000

// INIT flags
/* When using this flag in the ttos_init_system() function, the app asks to be the WindowMaster, able to send to background or
 * foreground other applications, and manage everything.
 */
#define TTOS_INIT_ASK_WM 1

// WIDGET flags
/* When a widget has the H_FIXED flag, it will remain static when a tab is scrolled horizontally by de user;
 * When a widget has the V_FIXED flag, it will remain static when a tab is scrolled vertically by de user;
 * When a widget has the FIXED flag, it will remain static when a tab is scrolled by de user, no matter the direction
 */
#define TTOS_WIDGET_H_FIXED 0x01
#define TTOS_WIDGET_V_FIXED 0x02
#define TTOS_WIDGET_FIXED (TTOS_WIDGET_V_FIXED | TTOS_WIDGET_H_FIXED)
/* When a widget has the CLICK_EVENT flag, it will process the click events done over it by the user, and send them to its
 * event port */
#define TTOS_WIDGET_CLICK_EVENT 0x04
/* When a widget has the DRAG_EVENT flag, it will process the drag events done over it by the user, and send them to its
 * event port */
#define TTOS_WIDGET_DRAG_EVENT 0x08
// TAB_OFFSET flags
// These flags are valid for ttos_set_tab_offset() function.
/* If DO_SCROLL is set, the tab will be progressively scrolled until it reaches the desired offset, and a TTOS_SR_SCROLL
 * event will bwe sent when the scroll ended; if not, the offset will be set instantaneously (and the event sent too).
 * The KEEP_HORIZONTAL flag ignores the specified horizontal offset, keeping the current offset of the tab
 * The KEEP_VERTICAL flag ignores the specified vertical offset, keeping the current offset of the tab
 * The SCREEN_SIZE_COORDS flag specifies that the offset coordinates are screen-sized coordinates; this is, an offset of 1,2, when
 * the screen resolution is 320x240, means that the offset will be 320,480.
 */
#define TTOS_TAB_OFFSET_DO_SCROLL 0x10
#define TTOS_TAB_OFFSET_KEEP_HORIZONTAL 0x20
#define TTOS_TAB_OFFSET_KEEP_VERTICAL 0x40
#define TTOS_TAB_OFFSET_SCREEN_SIZE_COORDS 0x80
// FONT flags
/* A label, by default, has only one line, and the text is cut at its end.
 * The FONT_MULTILINE flag specifies that the label will show several lines, cutting the lines exactly at the end of the
 * specified width. If the FONT_WORD_CUT flag is also set, the lines will be cut at the spaces between words.
 * By default, text is in UTF-8, unless FONT_LATIN1 flag is set
 * FONT_HALIGN_XXX specifies the horizontal alignment
 * FONT_VALIGN_XXX specifies the vertical alignment
 * FONT_ALIGN_CENTER is a shortcut for horizontal and vertical center
 */
#define TTOS_WIDGET_FONT_MULTILINE 0x100
#define TTOS_WIDGET_FONT_WORD_CUT 0x200
#define TTOS_WIDGET_FONT_LATIN1 0x400
#define TTOS_WIDGET_FONT_HALIGN_LEFT 0x0000
#define TTOS_WIDGET_FONT_HALIGN_CENTER 0x0800
#define TTOS_WIDGET_FONT_HALIGN_RIGHT 0x01000
#define TTOS_WIDGET_FONT_VALIGN_TOP 0x02000
#define TTOS_WIDGET_FONT_VALIGN_MIDDLE 0x0000
#define TTOS_WIDGET_FONT_VALIGN_BOTTOM 0x04000
#define TTOS_WIDGET_FONT_ALIGN_CENTER (TTOS_WIDGET_FONT_HALIGN_CENTER | TTOS_WIDGET_FONT_VALIGN_MIDDLE)
/* When using ttos_set_label_text(), by default the original size will be used, unless the FONT_UPDATE_SIZE is used, in which case
 * the new size will be the needed for the label to be fully shown
 */
#define TTOS_WIDGET_FONT_UPDATE_SIZE 0x08000

/* In a one-line, full-size label, by default, the text is cut at the right when it's too long, unless the BIG_LABEL_ALIGN_RIGHT
 * flag is used.
 */
#define TTOS_WIDGET_FONT_BIG_LABEL_ALIGN_LEFT 0x0000
#define TTOS_WIDGET_FONT_BIG_LABEL_ALIGN_RIGHT 0x010000
// BUTTON flags
/* This flag shows a stock button only with its icon, removing the text. Useful for small buttons */
#define TTOS_WIDGET_BUTTON_NO_TEXT 0x020000
// DRAWABLE flags
/* When creating a drawable object, this flag means that the caller wants a drawable with alpha channel */
#define TTOS_DRAWABLE_HAS_ALPHA  0x040000
// SET_CURRENT_TAB flags
/* When using ttos_set_visible_tab to change the currently visible tab, the SWAP_MODE_REPLACE just replaces the old one with the
 * new one directly, without any kind of animation. The flags SWAP_MODE_SCROLL_XXX allows to do the replacement with a scroll,
 * making the old tab to scroll out and the new tab scroll in.
 */
#define TTOS_TAB_SWAP_MODE_REPLACE      0x00
#define TTOS_TAB_SWAP_MODE_SCROLL_RIGHT 0x01
#define TTOS_TAB_SWAP_MODE_SCROLL_LEFT  0x02
#define TTOS_TAB_SWAP_MODE_SCROLL_UP    0x03
#define TTOS_TAB_SWAP_MODE_SCROLL_DOWN  0x04
// DESTROY TAB flag
/* If the currently visible tab is destroyed, the screen will become black because there is no visible tab until ttos_set_visible_tab
 * is used. But if the visible tab is destroyed with a call to ttos_destroy_tab that has this flag, then the destruction will be
 * delayed until the tab is no longer the visible tab. This allows to show a tab, destroys it inmediately (but will remain fully
 * functional), do the desired operations, and finally show a new tab. Only at that moment the old tab will be destroyed.
 * If this flag is used to destroy a non visible tab, it will be inmediately destroyed.
 */
#define TTOS_TAB_DELAY_DESTROY 0x08

//FILE selector flags
/* When creating a file selector, the FILE_SELECT_FOLDER will hide the files and show only the folders, to allow the user to choose
 * just a folder.
 * By default, the user will be able to navigate only in the original folder and its childrens, unless the ALLOW_GO_UP flag is set,
 * which will allow the user to go to /etc, /usr...
 * By default, the file selector doesn't show the hidden files, unless the SHOW_HIDDEN flag is set
 */
#define TTOS_FILE_SELECT_FOLDER 0x01
#define TTOS_FILE_ALLOW_GO_UP 0x02
#define TTOS_FILE_SHOW_HIDDEN 0x04

//IPC flags
/*
 * When creating a bus, it can be OWNER_TO_ALL (the creator sends a message and it is received by all the subscribers, but the
 * subscribers can't send messages to the bus) or ALL_TO_OWNER (the subscribers send a message and it is received only by the
 * creator). Only one of the two modes is allowed.
 */
#define TTOS_IPC_OWNER_TO_ALL 0x01
#define TTOS_IPC_ALL_TO_OWNER 0x02

enum ttos_file_type { TTOS_FTYPE_UNKNOWN, TTOS_FTYPE_END_LIST, TTOS_FTYPE_MP3, TTOS_FTYPE_VORBIS, TTOS_FTYPE_OGG, TTOS_FTYPE_APP };

struct ttos_color {
	uint8_t r,g,b,a; ///< Red, green and blue components, and Alpha channel value
};

struct ttos_pixel_format {
	uint8_t lshift; ///< rounds to shift to the left the color component value (after being shifted to the right)
	uint8_t rshift; ///< rounds to shift to the right the color component value (before being shifted to the left)
	uint32_t mask; ///< Mask value to apply to the color component after being shifted to right and left
};

struct ttos_window_geometry {
	uint16_t x; ///< X coordinate
	uint16_t y; ///< Y coordinate
	uint16_t w; ///< Width
	uint16_t h; ///< Height
	bool visible; ///< The window will be visible or not
};

struct ttos_object {

	struct minibus_connection *conn; ///< The connection to the window server
	struct minibus_master *queue; ///< The master queue
	void (*cb_port0)(struct ttos_object *master, uint32_t event, void *data); ///< The port0 callback
	void (*exit_cb)(struct ttos_object *master,void *data); ///< Exit callback
	void (*new_client) (struct ttos_object *master, uint32_t uid, Cstring icon, void *data); ///< New client event
	void (*destroyed_client) (struct ttos_object *master, uint32_t uid, uint32_t window, bool was_fg, void *data); ///< Destroyed client event
	void *cb_port0_data; ///< Private data for the port0 callback
	void *exit_cb_data; ///< Private data for the exit callback
	void *new_client_data; ///< Private data for the new client callback
	void *destroyed_client_data; ///< Private data for the destroyed client callback
	uint32_t uid; ///< Connection UID
	uint32_t ipc_uid; ///< IPC uid for bus communication
	uint32_t init_flags; ///< Flags accepted during startup
	int16_t ox; ///< X coordinate of the window assigned to this connection
	int16_t oy; ///< Y coordinate of the window assigned to this connection
	uint16_t width; ///< Screen width
	uint16_t height; ///< Screen height
	uint32_t shcounter; ///< counter for shared memory names
	bool fg; ///< True if the application is in foreground
	enum TTOS_PIXEL_FORMAT pixel_format; ///< Current pixel format (if is TTOS_PIXEL_FORMAT_OTHER, the only way to know it is to use the later info)
	uint8_t bytes_per_pixel; ///< bytes per pixel used by the current graphics mode
	struct ttos_pixel_format R_format; ///< pixel format for RED color for the current graphics mode
	struct ttos_pixel_format G_format; ///< pixel format for GREEN color for the current graphics mode
	struct ttos_pixel_format B_format; ///< pixel format for BLUE color for the current graphics mode
	struct ttos_pixel_format A_format; ///< pixel format for the ALPHA channel for the current graphics mode
};

struct ttos_dialog_button {
	enum ttos_stock_button button; ///< Stock button for this button
	Cstring text; ///< text for this button (only if the button field is TTOS_NO_STOCK_BUTTON)
	Cstring icon; ///< icon for this button (only if the button field is TTOS_NO_STOCK_BUTTON). Can be NULL
	uint16_t return_value; ///< Value to return if this button is pressed by the user
};

struct ttos_sharemem_object {
	uint32_t width; ///< Width in pixels
	uint32_t height; ///< Height in pixels
	bool has_alpha; ///< If true, this sharemem has alpha channel; if false, it has not
	void *pixel; ///< Pointer to the pixel memory zone
	void *alpha; ///< Pointer to the alpha memory zone. Valid only if has_alpha is true AND A_format in struct ttos_object has all its fields as zero; if not, the alpha channel is mixed with the pixels
	uint32_t uid; ///< Sharemem UID
	size_t size; ///< Size in bytes
};

#endif /* TTOS_STRUCTS_H_ */
