/*
 * Copyright 2011 Raster Software Vigo
 *
 * This file is part of ttos
 *
 * ttos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ttos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * widgets.h
 *
 *  Created on: 25/09/2010
 *      Author: raster
 */

#ifndef WIDGETS_H_
#define WIDGETS_H_

#include "minibus/minibus.h"
#include "bitmaps.h"
#include "tabs.h"
#include "portable.h"
#include "sharemem.h"

enum _widget_type {TTOS_WIDGET_PICTURE, TTOS_WIDGET_LABEL, TTOS_WIDGET_BUTTON, TTOS_WIDGET_AREA, TTOS_WIDGET_DRAWABLE};

/**
 * This structure defines a generic widget
 */

struct _widget {

	struct _tab *parent_tab; ///< Pointer to the tab into which this widget is
	bool (*event_cb) (struct ttos_tssample *, struct _widget *); ///< Callback for TouchScreen events
	void (*repaint_callback)(struct _widget *,int16_t x_offset,int16_t y_offset,struct _window *parent_window); ///< Callback for repaint the widget
	enum _widget_type type; ///< Widget type
	uint32_t widget_id; ///< The widget unique identifier
	int16_t ox; ///< left coordinate
	int16_t oy; ///< up coordinate
	int16_t w; ///< width
	int16_t h; ///< height
	bool v_fixed; ///< if true, the widget will remain fixed when vertically scrolling the window; if false, will scroll
	bool h_fixed; ///< if true, the widget will remain fixed when horizontally scrolling the window; if false, will scroll
	bool visible; ///< if true, the widget will be visible; if false, will be invisible
	bool enabled; ///< if true, the widget will respond to click and drag events; if false, it won't
	uint32_t event_port; ///< Port for events
	bool click_sensitive; ///< sensitive to click events
	bool drag_sensitive; ///< sensitive to drag events
};

struct _picture_widget {

	struct _widget; ///< parent "class"
	TTOS_Surface *bitmap; ///< Bitmap

};

struct _drawable_widget {
	struct _widget; ///< parent "class"
	TTOS_Surface *bitmap; ///< Bitmap
	bool shared_memory;
};

struct _label_widget {

	struct _widget; ///< parent "class"
	TTOS_Surface *bitmap; ///< Label
	int16_t o_x; ///< original x coordinate
	int16_t o_y; ///< original y coordinate
	uint16_t o_w; ///< original width
	uint16_t o_h; ///< original height
	int16_t px_x; ///< internal x offset
	int16_t px_y; ///< internal y offset
	struct ttos_color o_color; ///< original color

};

struct _button_widget {

	struct _widget; ///< parent "class"
	TTOS_Surface *bitmap; ///< Button

};

struct _area_widget {

	struct _widget; ///< parent "class"
	struct ttos_color color; ///< color components

};

void widgets_init_base_widget(struct _widget *object, struct _tab *parent_tab, uint32_t widget_id,enum _widget_type type);
void widgets_free_widget(void *element);
struct _widget *widgets_find_widget(struct _tab *tab, uint32_t widget_id);
uint32_t widgets_set_visibility(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
uint32_t widgets_set_position(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
uint32_t widgets_create_picture(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
uint32_t widgets_set_picture(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
uint32_t widgets_create_label(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
uint32_t widgets_set_label_text(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
uint32_t widgets_create_button(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
uint32_t widgets_create_color_area(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
uint32_t widgets_create_drawable(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
uint32_t widgets_update_drawable(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);

#endif /* WIDGETS_H_ */
